/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of the HDF NPP Aggregation Tool (nagg). The full        *
 * NPP Aggregation Tool copyright notice, including terms governing use,     *
 * modification, and redistribution, is contained in the file COPYING,       *
 * which can be found at the root of the source code distribution tree.      *
 * If you do not have access to this file, you may request a copy from       *
 * help@hdfgroup.org.                                                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* This holds the definition of the Product_table and its access
 * functions.
 */

#include <nagg.h>
/* Public variable definition */

equivalentgeoproducts_t equiv_geoprod_table[NPP_EQ_GEO_Products_max] =
{
/* GPID     TCPID  */
"GATRO",    "GCRIO",
"GIMGO",    "GITCO",
"GMODO",    "GMTCO"
} ;

/* NPP Products Table
 * The source is NPOESS Common Data Format Control Book Volume I, pp 173-321,
 * Tables A-2 - A-7, Data Record Identifiers and Data Mapping.
 * column 1: DPID
 * column 2: Product Short Name
 * column 3: Granule Nominal Duration (miscroseconds). 0 means unknown.
 *           Granule durations extracted from Raytheon's INF_CFG.xml DDS 
 *           configuration file.
 * column 4: Geolocation product ID used by this product.
 * Note: any duration value larger than 2**31 (~2 billion) should have a
 * ULL qualifier to avoid constant overflow.
 *
 * 22 Intermediate products from JPSS Internal Data Format Control Book 
 * Volume III, Appendix A added to product_table in version 1.5.1. 
 */
nppproduct_t product_table[NPP_Product_max] =
{
/* DPID         Short Name                              Duration 	GPID  */
"ICALI",        "CrIMSS-CrIS-AVMP-LOS-IR-IP",           31997000, 	"GCRIO",
"ICALM",        "CrIMSS-CrIS-AVMP-LOS-MW-IP",         	31997000, 	"GCRIO",
"ICCCR",        "CrIMSS-CrIS-CLOUD-CLEARED-RAD-IP",	31997000, 	"GCRIO",
"ICISE",        "CrIMSS-CrIS-IR-SURF-EMISSIVITY-IP",    31997000,  	"GCRIO",
"ICMSE",        "CrIMSS-CrIS-MW-SURF-EMISSIVITY-IP",    31997000,  	"GCRIO",
"ICSTT",        "CrIMSS-CrIS-SKIN-TEMP-IP",             31997000,  	"GCRIO",
"ICTLI",        "CrIMSS-CrIS-AVTP-LOS-IR-IP",           31997000,  	"GCRIO",
"ICTLM",        "CrIMSS-CrIS-AVTP-LOS-MW-IP",           31997000,  	"GCRIO",
"IICMO",        "VIIRS-CM-IP",                          85350000,  	"GMODO",
"IICMS",        "VIIRS-CM-IP-SUB",                      85350000,  	"GMODO",
"SATMR",        "ATMS-REMAP-SDR",        		31997000,	"GATRO",
"SATMS",        "ATMS-SDR",                        	31997000, 	"GATMO",
"SCRIS",        "CrIS-SDR",                             31997000, 	"GCRSO",
"SOMPS",        "OMPS-NP-SDR",                	        37405000, 	"GONPO",
"SOMTC",        "OMPS-TC-SDR",                 	        37405000, 	"GOTCO",
"SOMSC",        "OMPS-TC-Cal-SDR",     		        2700000000ULL, 	"GOSCO",
"SOMNC",        "OMPS-NP-Cal-SDR",   		        2700000000ULL, 	"GONCO",
"SVDNB",        "VIIRS-DNB-SDR",                	85350000,       "GDNBO",
"SVI01",        "VIIRS-I1-SDR",                       	85350000, 	"GIMGO",
"SVI02",        "VIIRS-I2-SDR",             	        85350000, 	"GIMGO",
"SVI03",        "VIIRS-I3-SDR",                       	85350000, 	"GIMGO",
"SVI04",        "VIIRS-I4-SDR",                       	85350000, 	"GIMGO",
"SVI05",        "VIIRS-I5-SDR",                       	85350000, 	"GIMGO",
"SVM01",        "VIIRS-M1-SDR",                	 	85350000,	"GMODO",
"SVM02",        "VIIRS-M2-SDR",                 	85350000,	"GMODO",
"SVM03",        "VIIRS-M3-SDR",                 	85350000,	"GMODO",
"SVM04",        "VIIRS-M4-SDR",                 	85350000,	"GMODO",
"SVM05",        "VIIRS-M5-SDR",                 	85350000,	"GMODO",
"SVM06",        "VIIRS-M6-SDR",                 	85350000,	"GMODO",
"SVM07",        "VIIRS-M7-SDR",                 	85350000,	"GMODO",
"SVM08",        "VIIRS-M8-SDR",                 	85350000,	"GMODO",
"SVM09",        "VIIRS-M9-SDR",                 	85350000,	"GMODO",
"SVM10",        "VIIRS-M10-SDR",            	 	85350000,	"GMODO",
"SVM11",        "VIIRS-M11-SDR",               	       	85350000,	"GMODO",
"SVM12",        "VIIRS-M12-SDR",              		85350000,	"GMODO",
"SVM13",        "VIIRS-M13-SDR",              		85350000,	"GMODO",
"SVM14",        "VIIRS-M14-SDR",               	       	85350000,	"GMODO",
"SVM15",        "VIIRS-M15-SDR",               	       	85350000,	"GMODO",
"SVM16",        "VIIRS-M16-SDR",               	       	85350000,	"GMODO",
"TATMS",        "ATMS-TDR",                        	31997000, 	"GATMO",
"REDRO",        "CrIMSS-EDR",                      	31997000, 	"GCRIO",
"OOTCO",        "OMPS-TC-EDR",                 	       	37405000, 	"GOTCO",
"VAOOO",        "VIIRS-Aeros-EDR",            	       	85350000,	"GAERO",
"VCBHO",        "VIIRS-CBH-EDR",                	85350000,	"GCLDO",
"VCCLO",        "VIIRS-CCL-EDR",                  	85350000,	"GCLDO",
"VCEPO",        "VIIRS-CEPS-EDR",               	85350000,	"GCLDO",
"VCOTO",        "VIIRS-COT-EDR",                	85350000,	"GCLDO",
"VCTHO",        "VIIRS-CTH-EDR",                	85350000,	"GCLDO",
"VCTPO",        "VIIRS-CTP-EDR",                 	85350000,	"GCLDO",
"VCTTO",        "VIIRS-CTT-EDR",                 	85350000,	"GCLDO",
"VI1BO",        "VIIRS-I1-IMG-EDR",            		85350000,	"GIGTO",
"VI2BO",        "VIIRS-I2-IMG-EDR",            		85350000,	"GIGTO",
"VI3BO",        "VIIRS-I3-IMG-EDR",            		85350000,	"GIGTO",
"VI4BO",        "VIIRS-I4-IMG-EDR",            		85350000,	"GIGTO",
"VI5BO",        "VIIRS-I5-IMG-EDR",            		85350000,	"GIGTO",
"VISTO",        "VIIRS-IST-EDR",                    	85350000,	"GMTCO",
"VLSTO",        "VIIRS-LST-EDR",                  	85350000,	"GMTCO",
"VM01O",        "VIIRS-M1ST-EDR",            		85350000,	"GMGTO",
"VM02O",        "VIIRS-M2ND-EDR",           	       	85350000,	"GMGTO",
"VM03O",        "VIIRS-M3RD-EDR",           		85350000,	"GMGTO",
"VM04O",        "VIIRS-M4TH-EDR",           		85350000,	"GMGTO",
"VM05O",        "VIIRS-M5TH-EDR",           		85350000,	"GMGTO",
"VM06O",        "VIIRS-M6TH-EDR",           		85350000,	"GMGTO",
"VNCCO",        "VIIRS-NCC-EDR",               		85350000,	"GNCCO",
"VNHFO",        "VIIRS-NHF-EDR",               		85350000,	"GNHFO",
"VOCCO",        "VIIRS-OCC-EDR",               		85350000,	"GMTCO",
"VISAO",        "VIIRS-SA-EDR",                    	85350000,	"GMTCO",
"VSCDO",        "VIIRS-SCD-BINARY-SNOW-FRAC-EDR",     	85350000,	"GMTCO",
"VSCMO",        "VIIRS-SCD-BINARY-SNOW-MAP-EDR",     	85350000,	"GITCO",
"VSICO",        "VIIRS-SIC-EDR",                   	85350000,	"GMTCO",
"VSSTO",        "VIIRS-SST-EDR",                 	85350000,	"GMTCO",
"VSTYO",        "VIIRS-ST-EDR",                   	85350000,	"GMTCO",
"VSUMO",        "VIIRS-SusMat-EDR",       		85350000,	"GMTCO",
"VIVIO",        "VIIRS-VI-EDR",                     	85350000,	"GITCO",
"REDRS",        "CrIMSS-EDR-SUB",            		31997000,	"GCRIO",
"OOTCS",        "OMPS-TC-EDR-SUB",        		37405000, 	"GOTCO",
"VAOOS",        "VIIRS-Aeros-EDR-SUB",    	        85350000,	"GAERO",
"VCBHS",        "VIIRS-CBH-EDR-SUB",        	        85350000,	"GCLDO",
"VCCLS",        "VIIRS-CCL-EDR-SUB",          	        85350000,	"GCLDO",
"VCEPS",        "VIIRS-CEPS-EDR-SUB",       		85350000,	"GCLDO",
"VCOTS",        "VIIRS-COT-EDR-SUB",       		85350000,	"GCLDO",
"VCTHS",        "VIIRS-CTH-EDR-SUB",       		85350000,	"GCLDO",
"VCTPS",        "VIIRS-CTP-EDR-SUB",       	 	85350000,	"GCLDO",
"VCTTS",        "VIIRS-CTT-EDR-SUB",        		85350000,	"GCLDO",
"VISTS",        "VIIRS-IST-EDR-SUB",          		85350000,	"GMTCO",
"VLSTS",        "VIIRS-LST-EDR-SUB",         		85350000,	"GMTCO",
"VNCCS",        "VIIRS-NCC-EDR-SUB",      		85350000,	"GNCCO",
"VNHFS",        "VIIRS-NHF-EDR-SUB",      		85350000,	"GNHFO",
"VOCCS",        "VIIRS-OCC-EDR-SUB",     	 	85350000,	"GMTCO",
"VISAS",        "VIIRS-SA-EDR-SUB",           		85350000,	"GMTCO",
"VSCDS",        "VIIRS-SCD-BINARY-SNOW-FRAC-EDR-SUB",   85350000,	"GMTCO",
"VSCMS",        "VIIRS-SCD-BINARY-SNOW-MAP-EDR-SUB",    85350000,	"GITCO",
"VSICS",        "VIIRS-SIC-EDR-SUB",           		85350000,	"GMTCO",
"VSSTS",        "VIIRS-SST-EDR-SUB",         		85350000,	"GMTCO",
"VSTPS",        "VIIRS-ST-EDR-SUB",           		85350000,	"GMTCO",
"VSUMS",        "VIIRS-SusMat-EDR-SUB",  		85350000,	"GMTCO",
"VIVIS",        "VIIRS-VI-EDR-SUB",              	85350000,	"GITCO",
"INCTO",        "OMPS-TC-Oz-Fst-Guess-IP",              37405000,       "GOTCO",
"INPAK",        "OMPS-NP-Ave-Ker-IP",                   37405000,       "GONPO",
"IIROO",        "CrIS-IROZ-Prof-IP",                    31997000,       "GCRIO",
"IIROS",        "CrIS-IROZ-Prof-IP-SUB",                31997000,       "GCRIO",
"IMOPO",        "OMPS-NP-IP",                           37405000,       "GONPO",
"IMOPS",        "OMPS-NP-IP-SUB",                       37405000,       "GONPO",
"IVAMI",        "VIIRS-Aeros-Modl-Info-IP",             85350000,       "GMTCO",
"IVAOT",        "VIIRS-Aeros-Opt-Thick-IP",             85350000,       "GMTCO",
"IVBPX",        "VIIRS-Bright-Pixel-Mod-IP",            85350000,       "GMODO",
"IVCBH",        "VIIRS-CB-Ht-IP",                       85350000,       "GMODO",
"IVCDB",	"VIIRS-DualGain-Cal-IP",                85350000,       "ICDBG",
"IVCLT",        "VIIRS-Cd-Cov-Type-IP",                 85350000,       "GCLDO",
"IVCOP",        "VIIRS-Cd-Opt-Prop-IP",                 85350000,       "GMODO",
"IVCTP",	"VIIRS-Cd-Top-Parm-IP",                 85350000,       "GMODO",
"IVICC",        "VIIRS-Cd-Layer-Type-IP",               85350000,       "GMODO",
"IVIIC",        "VIIRS-I-Conc-IP",                      85350000,       "GITCO",
"IVIIW",        "VIIRS-I-Wts-IP",                       85350000,       "GITCO",
"IVIQF",        "VIIRS-I-Qual-Flags-IP",                85350000,       "GITCO",
"IVIRT",        "VIIRS-I-Refl-Temp-IP",                 85350000,       "GITCO",
"IVISR",        "VIIRS-Surf-Refl-IP",                   85350000,       "GMTCO",
"IVIWT",        "VIIRS-INWCTT-IP",                      85350000,       "GMODO",
"IVPCM",        "VIIRS-Parx-Corr-CM-IP",                85350000,       "GMODO",
"IVPCP",        "VIIRS-Parx-Corr-Cd-Opt-Prop-IP",       85350000,       "GMODO",
"IVPTP",        "VIIRS-Parx-Corr-Cd-Top-Parm-IP",       85350000,       "GMODO",
"IVSIC",        "VIIRS-GridIP-VIIRS-Snow-Ice-Cover-Mod-Gran",85350000,  "GMTCO",
"IVSTP",        "VIIRS-Surf-Temp-IP",                    85350000,       "GITCO"
} ;


/* NPP Geolocation Table
 * The source is NPOESS Common Data Format Control Book Volume I, pp 328-9,
 * Table A-8, Geolocation Identifiers.
 * column 1: DPID
 * column 2: Product Short Name
 * column 3: Granule Nominal Duration (miscroseconds).  Granule durations
 *           extracted from Raytheon's INF_CFG.xml DDS configuration file.
 * column 4: Geolocation product ID used by this product. Is always NULL.
 * Note: any duration value larger than 2**31 (~2 billion) should have a
 * ULL qualifier to avoid constant overflow. 
 *
 * 1 geolocation product from JPSS Internal Data Format Control Book
 * Volume III, Appendix A added to geolocation_table in version 1.5.1. 
 */
nppproduct_t geolocation_table[NPP_Geo_Location_max]=
{
/* DPID         Short Name                              Duration 	GPID  */
"GATMO",        "ATMS-SDR-GEO",                   	31997000,	NULL,
"GCRSO",        "CrIS-SDR-GEO",                        	31997000,	NULL,
"GAERO",        "VIIRS-Aeros-EDR-GEO",          	85350000,	NULL,
"GCLDO",        "VIIRS-CLD-AGG-GEO",             	85350000,	NULL,
"GDNBO",        "VIIRS-DNB-GEO",                    	85350000,	NULL,
"GNCCO",        "VIIRS-NCC-EDR-GEO",            	85350000,	NULL,
"GIGTO",        "VIIRS-IMG-GTM-EDR-GEO",   		85350000,	NULL,
"GIMGO",        "VIIRS-IMG-GEO",                    	85350000,	NULL,
"GITCO",        "VIIRS-IMG-GEO-TC",                	85350000,	NULL,
"GMGTO",        "VIIRS-MOD-GTM-EDR-GEO",       		85350000,	NULL,
"GMODO",        "VIIRS-MOD-GEO",                	85350000,	NULL,
"GMTCO",        "VIIRS-MOD-GEO-TC",           		85350000,	NULL,
"GNHFO",        "VIIRS-NHF-EDR-GEO",          		85350000,	NULL,
"GOTCO",        "OMPS-TC-GEO",                    	37405000, 	NULL,
"GOSCO",        "OMPS-TC-Cal-GEO",             	        2700000000ULL, 	NULL,
"GONPO",        "OMPS-NP-GEO",                  	37405000,  	NULL,
"GONCO",        "OMPS-NP-Cal-GEO",           	        2700000000ULL, 	NULL,
"GCRIO",        "CrIMSS-EDR-GEO-TC",          		31997000,	NULL,
"GATRO",        "ATMS-REMAP-SDR-GEO",   		31997000,	NULL,
"ICDBG",        "VIIRS-MOD-UNAGG-GEO",                  85350000,       NULL
};

/* local macro definitions */

/* Local variables definitions */
/* cache last search for quicker return for repeated search */
static int last_found=0;
static int last_geo_found=0;


/* function definitions */

/* Get Product Short name by Product DPID.
 * prod_id: Product DPID
 * Return: char* of the Product Short name if found;
 *         NULL otherwise.
 */
char *
get_product_sname_by_id(const char *prod_id) 
{
    char *return_val=NULL;
    int i;

    /* arugment check */
    assert(prod_id!=NULL);

    /* Is this a repeat search? */
    if (HDstreq(prod_id, product_table[last_found].DPID)){
	return product_table[last_found].sname;
    }
    if (HDstreq(prod_id, geolocation_table[last_geo_found].DPID)){
	return geolocation_table[last_geo_found].sname;
    }

    /* search data product table first, then geo-product table */
    for (i=0; i<NPP_Product_max; ++i) {
        if (HDstreq(prod_id, product_table[i].DPID)){
	    last_found=i;
	    return product_table[i].sname;
	}
    }

    /* search geo-proudct table too */
    for (i=0; i<NPP_Geo_Location_max; ++i) {
        if (HDstreq(prod_id, geolocation_table[i].DPID)){
	    last_geo_found=i;
	    return geolocation_table[i].sname;
	}
    }

    return return_val;
}

/* Get Product DPID by Product Short name.
 * prod_sname: Product Short name
 * Return: char* of the Product ID if found;
 *         NULL otherwise.
 */
char *
get_product_id_by_sname(const char *prod_sname) 
{
    char *return_val=NULL;
    int i;

    /* arugment check */
    assert(prod_sname!=NULL);

    /* Is this a repeat search? */
    if (HDstreq(prod_sname, product_table[last_found].sname)){
	return product_table[last_found].DPID;
    }
    if (HDstreq(prod_sname, geolocation_table[last_geo_found].sname)){
	return geolocation_table[last_geo_found].DPID;
    }

    /* search data product table first, then geo-product table */
    for (i=0; i<NPP_Product_max; ++i) {
        if (HDstreq(prod_sname, product_table[i].sname)){
	    last_found=i;
	    return product_table[i].DPID;
	}
    }

    /* search geo-proudct table too */
    for (i=0; i<NPP_Geo_Location_max; ++i) {
        if (HDstreq(prod_sname, geolocation_table[i].sname)){
	    last_geo_found=i;
	    return geolocation_table[i].DPID;
	}
    }

    return return_val;
}

/* Get Geoproduct ID by Product DPID.
 * prod_id: Product DPID
 * Return: char* of the Geoproduct ID if found;
 *         NULL otherwise.
 */
char *
get_gpid_by_id(const char *prod_id)
{
    char *return_val=NULL;
    int i;

    /* arugment check */
    assert(prod_id!=NULL);

    /* No repeat search optimization like the other functions since
     * it is not likely a repeat product search.
     */

    /* search data product table only since geo-product has no gpid. */
    for (i=0; i<NPP_Product_max; ++i) {
        if (HDstreq(prod_id, product_table[i].DPID)){
	    return product_table[i].GPID;
	}
    }

    return return_val;
}

/* Get Terrain Corrected Geoproduct ID by Geoproduct DPID.
 * gpid: Geoproduct GPID
 * Return: char* of the Terrain Corrected Geoproduct ID if found;
 *         NULL otherwise.
 */
char *
get_tcpid_by_gpid(const char *gpid)
{
    char *return_val=NULL;
    int i;

    /* arugment check */
    assert(gpid!=NULL);

    /* No repeat search optimization like the other functions since
     * it is not likely a repeat product search.
     */

    /* search data product table only since geo-product has no gpid. */
    for (i=0; i<NPP_EQ_GEO_Products_max; ++i) {
        if (HDstreq(gpid, equiv_geoprod_table[i].GPID)){
	    return equiv_geoprod_table[i].TCPID;
	}
    }

    return return_val;
}

/* Display the NPP sensor and geolocation product lists.
 * Return: no return value.
 */
void
print_product_lists(void)
{
    int i;

    /* Sensor product table */
    printf( "Equivalent GPIDs\n"
        "GPID      TCPID\n"
        "-----     ------\n"
        "GATRO     GCRIO\n"
        "GIMGO     GITCO\n"
        "GMODO     GMTCO\n\n"

        "Products having the same GPID are considered compatible and can be packaged\n"
        "together.  In addition, three pairs of GPIDs are related by terrain correction\n"
        "and are considered equivalent for the purpose of determining compatibility.\n"
        "Any product with a GPID in the table of Equivalent GPIDs above is also\n"
        "compatible with any product whose GPID is in the same row of the table.\n\n"
         
        "NPP Product list\n"
	"DPID      Short Name\t\t\t\t  Duration  GPID\n"
	"----      ----------\t\t\t\t---------- ------\n");
    for (i=0; i<NPP_Product_max; i++){
	printf("%s     %-32s\t%10lld %6s\n", 
	    product_table[i].DPID,
	    product_table[i].sname,
	    product_table[i].duration,
	    product_table[i].GPID);
    }
    printf("\n");

    /* Geolocation product table */
    printf("NPP Geolocation Product list\n"
	"DPID      Short Name\t\t\t\tDuration\n"
	"----      ----------\t\t\t\t--------\n");
    for (i=0; i<NPP_Geo_Location_max; i++){
	printf("%s     %-32s\t%10lld\n", 
	    geolocation_table[i].DPID,
	    geolocation_table[i].sname,
	    geolocation_table[i].duration);
    }
}

/* Get Product duration by Product DPID.
 * prod_id: Product DPID
 * Return: inet_t of duration which is always positive.
 *         0 otherwise.
 */
iet_t
get_product_duration_by_id(const char *prod_id) 
{
    iet_t return_val=0;
    int i;

    /* arugment check */
    assert(prod_id!=NULL);

    /* Is this a repeat search? */
    if (HDstreq(prod_id, product_table[last_found].DPID)){
	return product_table[last_found].duration;
    }
    if (HDstreq(prod_id, geolocation_table[last_geo_found].DPID)){
	return geolocation_table[last_geo_found].duration;
    }

    /* search data product table first, then geo-product table */
    for (i=0; i<NPP_Product_max; ++i) {
        if (HDstreq(prod_id, product_table[i].DPID)){
	    last_found=i;
	    return product_table[i].duration;
	}
    }

    /* search geo-proudct table too */
    for (i=0; i<NPP_Geo_Location_max; ++i) {
        if (HDstreq(prod_id, geolocation_table[i].DPID)){
	    last_geo_found=i;
	    return geolocation_table[i].duration;
	}
    }

    return return_val;
}

